/**
 * FF Typescript Foundation Library
 * Copyright 2021 Ralph Wiedemeier, Frame Factory GmbH
 *
 * License: MIT
 */
import { ITypedEvent } from "@ff/core/Publisher";
export declare type PointerEventType = "pointer-down" | "pointer-up" | "pointer-hover" | "pointer-move";
export declare type TriggerEventType = "wheel" | "double-click" | "context-menu" | "drag-enter" | "drag-over" | "drag-leave" | "drag-end" | "drop";
export declare type PointerEventSource = "mouse" | "pen" | "touch";
export interface IPointer {
    id: number;
    offsetX: number;
    offsetY: number;
}
export interface IManipEvent extends ITypedEvent<PointerEventType | TriggerEventType> {
    originalEvent: Event;
    /** The x coordinate of the center of all active pointers. */
    centerX: number;
    /** The y coordinate of the center of all active pointers. */
    centerY: number;
    /** True if the shift modifier is active. */
    shiftKey: boolean;
    /** True if the control modifier is active. */
    ctrlKey: boolean;
    /** True if the alt modifier is active. */
    altKey: boolean;
    /** True if the meta modifier is active. */
    metaKey: boolean;
    /** Set this to true to stop the event from further propagating. */
    stopPropagation: boolean;
}
export interface IPointerEvent extends IManipEvent {
    type: PointerEventType;
    originalEvent: PointerEvent;
    source: PointerEventSource;
    primaryButton: number;
    isPrimary: boolean;
    isDragging: boolean;
    pointerCount: number;
    activePointers: IPointer[];
    movementX: number;
    movementY: number;
}
export interface ITriggerEvent extends IManipEvent {
    type: TriggerEventType;
    wheel: number;
}
export interface IManipListener {
    onPointer: (event: IPointerEvent) => boolean;
    onTrigger: (event: ITriggerEvent) => boolean;
}
/**
 * Composable class, listens for mouse and touch events on its target and converts
 * them to [[IPointerEvent]] and [[ITriggerEvent]] events. [[IManip]] receivers of these events
 * can be chained. Events are handed down the chain, starting with the [[IManip]] instance
 * assigned to [[ManipTarget.next]].
 */
export default class ManipTarget {
    listener: IManipListener;
    protected activePointers: IPointer[];
    protected activeType: string;
    protected primaryButton: number;
    protected centerX: number;
    protected centerY: number;
    protected startX: number;
    protected startY: number;
    protected isDragging: boolean;
    constructor(target?: HTMLElement);
    onPointerDown(event: PointerEvent): void;
    onPointerMove(event: PointerEvent): void;
    onPointerUpOrCancel(event: PointerEvent): void;
    onDragEnter(event: DragEvent): void;
    onDragOver(event: DragEvent): void;
    onDragLeave(event: DragEvent): void;
    onDragEnd(event: DragEvent): void;
    onDrop(event: DragEvent): void;
    onDoubleClick(event: MouseEvent): void;
    onContextMenu(event: MouseEvent): void;
    onWheel(event: WheelEvent): void;
    protected createManipPointerEvent(event: PointerEvent, type: PointerEventType): IPointerEvent;
    protected createManipTriggerEvent(event: MouseEvent, type: TriggerEventType): ITriggerEvent;
    protected sendPointerEvent(event: IPointerEvent): boolean;
    protected sendTriggerEvent(event: ITriggerEvent): boolean;
}
