"use strict";
/**
 * FF Typescript Foundation Library
 * Copyright 2021 Ralph Wiedemeier, Frame Factory GmbH
 *
 * License: MIT
 */
Object.defineProperty(exports, "__esModule", { value: true });
////////////////////////////////////////////////////////////////////////////////
const _DRAG_DISTANCE = 2;
/**
 * Composable class, listens for mouse and touch events on its target and converts
 * them to [[IPointerEvent]] and [[ITriggerEvent]] events. [[IManip]] receivers of these events
 * can be chained. Events are handed down the chain, starting with the [[IManip]] instance
 * assigned to [[ManipTarget.next]].
 */
class ManipTarget {
    constructor(target) {
        this.listener = null;
        this.activePointers = [];
        this.activeType = "";
        this.primaryButton = 0;
        this.centerX = 0;
        this.centerY = 0;
        this.startX = 0;
        this.startY = 0;
        this.isDragging = false;
        this.onPointerDown = this.onPointerDown.bind(this);
        this.onPointerMove = this.onPointerMove.bind(this);
        this.onPointerUpOrCancel = this.onPointerUpOrCancel.bind(this);
        this.onDragEnter = this.onDragEnter.bind(this);
        this.onDragOver = this.onDragOver.bind(this);
        this.onDragLeave = this.onDragLeave.bind(this);
        this.onDragEnd = this.onDragEnd.bind(this);
        this.onDrop = this.onDrop.bind(this);
        this.onDoubleClick = this.onDoubleClick.bind(this);
        this.onContextMenu = this.onContextMenu.bind(this);
        this.onWheel = this.onWheel.bind(this);
        if (target) {
            target.addEventListener("pointerdown", this.onPointerDown);
            target.addEventListener("pointermove", this.onPointerMove);
            target.addEventListener("pointerup", this.onPointerUpOrCancel);
            target.addEventListener("pointercancel", this.onPointerUpOrCancel);
            target.addEventListener("dragenter", this.onDragEnter);
            target.addEventListener("dragover", this.onDragOver);
            target.addEventListener("dragleave", this.onDragLeave);
            target.addEventListener("dragend", this.onDragEnd);
            target.addEventListener("drop", this.onDrop);
            target.addEventListener("doubleclick", this.onDoubleClick);
            target.addEventListener("contextmenu", this.onContextMenu);
            target.addEventListener("wheel", this.onWheel);
        }
    }
    onPointerDown(event) {
        // only events of a single pointer type can be handled at a time
        if (this.activeType && event.pointerType !== this.activeType) {
            return;
        }
        if (event.isPrimary) {
            this.startX = event.offsetX;
            this.startY = event.offsetY;
            this.isDragging = false;
            this.primaryButton = event.button || 0;
        }
        this.activeType = event.pointerType;
        this.activePointers.push({
            id: event.pointerId,
            offsetX: event.offsetX,
            offsetY: event.offsetY
        });
        event.currentTarget.setPointerCapture(event.pointerId);
        const manipEvent = this.createManipPointerEvent(event, "pointer-down");
        if (this.sendPointerEvent(manipEvent)) {
            event.stopPropagation();
        }
        event.preventDefault();
    }
    onPointerMove(event) {
        const activePointers = this.activePointers;
        for (let i = 0, n = activePointers.length; i < n; ++i) {
            const pointer = activePointers[i];
            if (event.pointerId === pointer.id) {
                pointer.offsetX = event.offsetX;
                pointer.offsetY = event.offsetY;
            }
        }
        if (event.isPrimary && activePointers.length > 0 && !this.isDragging) {
            const delta = Math.abs(event.offsetX - this.startX) + Math.abs(event.offsetY - this.startY);
            if (delta > _DRAG_DISTANCE) {
                this.isDragging = true;
            }
        }
        const eventType = activePointers.length ? "pointer-move" : "pointer-hover";
        const manipEvent = this.createManipPointerEvent(event, eventType);
        if (this.sendPointerEvent(manipEvent)) {
            event.stopPropagation();
        }
        event.preventDefault();
    }
    onPointerUpOrCancel(event) {
        const activePointers = this.activePointers;
        let found = false;
        for (let i = 0, n = activePointers.length; i < n; ++i) {
            if (event.pointerId === activePointers[i].id) {
                activePointers.splice(i, 1);
                found = true;
                break;
            }
        }
        if (!found) {
            console.warn("[ManipTarget.onPointerUpOrCancel] orphan pointer up/cancel event #id", event.pointerId);
            return;
        }
        const manipEvent = this.createManipPointerEvent(event, "pointer-up");
        if (activePointers.length === 0) {
            this.activeType = "";
        }
        if (event.isPrimary) {
            this.primaryButton = 0;
            this.isDragging = false;
        }
        if (this.sendPointerEvent(manipEvent)) {
            event.stopPropagation();
        }
        event.preventDefault();
    }
    onDragEnter(event) {
        const consumed = this.sendTriggerEvent(this.createManipTriggerEvent(event, "drag-enter"));
        if (consumed) {
            event.preventDefault();
        }
    }
    onDragOver(event) {
        const consumed = this.sendTriggerEvent(this.createManipTriggerEvent(event, "drag-over"));
        if (consumed) {
            event.preventDefault();
        }
    }
    onDragLeave(event) {
        const consumed = this.sendTriggerEvent(this.createManipTriggerEvent(event, "drag-leave"));
        if (consumed) {
            event.preventDefault();
        }
    }
    onDragEnd(event) {
        const consumed = this.sendTriggerEvent(this.createManipTriggerEvent(event, "drag-end"));
        if (consumed) {
            event.preventDefault();
        }
    }
    onDrop(event) {
        const consumed = this.sendTriggerEvent(this.createManipTriggerEvent(event, "drop"));
        if (consumed) {
            event.preventDefault();
        }
    }
    onDoubleClick(event) {
        const consumed = this.sendTriggerEvent(this.createManipTriggerEvent(event, "double-click"));
        if (consumed) {
            event.preventDefault();
        }
    }
    onContextMenu(event) {
        this.sendTriggerEvent(this.createManipTriggerEvent(event, "context-menu"));
        // prevent default context menu regardless of whether event was consumed or not
        event.preventDefault();
    }
    onWheel(event) {
        const consumed = this.sendTriggerEvent(this.createManipTriggerEvent(event, "wheel"));
        if (consumed) {
            event.preventDefault();
        }
    }
    createManipPointerEvent(event, type) {
        // calculate center and movement
        let centerX = 0;
        let centerY = 0;
        let movementX = 0;
        let movementY = 0;
        const pointers = this.activePointers;
        const count = pointers.length;
        if (count > 0) {
            for (let i = 0; i < count; ++i) {
                centerX += pointers[i].offsetX;
                centerY += pointers[i].offsetY;
            }
            centerX /= count;
            centerY /= count;
            if (type === "pointer-move" || type === "pointer-hover") {
                movementX = centerX - this.centerX;
                movementY = centerY - this.centerY;
            }
            this.centerX = centerX;
            this.centerY = centerY;
        }
        else {
            centerX = event.offsetX;
            centerY = event.offsetY;
        }
        return {
            originalEvent: event,
            type: type,
            source: event.pointerType,
            primaryButton: this.primaryButton,
            isPrimary: event.isPrimary,
            isDragging: this.isDragging,
            activePointers: pointers,
            pointerCount: count,
            centerX,
            centerY,
            movementX,
            movementY,
            shiftKey: event.shiftKey,
            ctrlKey: event.ctrlKey,
            altKey: event.altKey,
            metaKey: event.metaKey,
            stopPropagation: false,
        };
    }
    createManipTriggerEvent(event, type) {
        let wheel = 0;
        if (type === "wheel") {
            wheel = event.deltaY;
        }
        return {
            originalEvent: event,
            type,
            wheel,
            centerX: event.offsetX,
            centerY: event.offsetY,
            shiftKey: event.shiftKey,
            ctrlKey: event.ctrlKey,
            altKey: event.altKey,
            metaKey: event.metaKey,
            stopPropagation: false,
        };
    }
    sendPointerEvent(event) {
        return this.listener && this.listener.onPointer(event);
    }
    sendTriggerEvent(event) {
        return this.listener && this.listener.onTrigger(event);
    }
}
exports.default = ManipTarget;
//# sourceMappingURL=ManipTarget.js.map